#include <mega32.h>
	#ifndef __SLEEP_DEFINED__
	#define __SLEEP_DEFINED__
	.EQU __se_bit=0x80
	.EQU __sm_mask=0x70
	.EQU __sm_powerdown=0x20
	.EQU __sm_powersave=0x30
	.EQU __sm_standby=0x60
	.EQU __sm_ext_standby=0x70
	.EQU __sm_adc_noise_red=0x10
	.SET power_ctrl_reg=mcucr
	#endif

#include <stdio.h>
#include <delay.h>


//========================================================================================================
#include <macros.h>
#include <sym.h>
#include <config.h>
#include <driver.h>
#include <driver.c>
/*   .     */
byte ReadCode( void ){
    byte ret;
    //   
    LCD_DATA_PORT=0x00;
    LCD_DATA_DDR=0x00;
    //      
    ClrBit(LCD_CONTROL_PORT, A0);
    SetBit(LCD_CONTROL_PORT, RDWR);
    //      
    delay_us(10);
    SetBit(LCD_CONTROL_PORT, E);
    //      
    delay_us(10);
    ret=LCD_DATA_PIN;
    //     
    ClrBit(LCD_CONTROL_PORT,E);
    return ret;
}


/*    */
void WaitBusy(void){
    byte stat;
    //      BUSY
    stat=ReadCode();
    while(stat==(1<<BUSY)) stat=ReadCode();
}


//    RESET
void WaitReset(void){
    byte stat;
    stat=ReadCode();
    while(stat==(1<<RESET)) stat=ReadCode();
}


//    ON/OFF
void WaitON(void){
    byte stat;
    stat=ReadCode();
    while(stat==(1<<ONOFF)) stat=ReadCode();
}


/*     */
byte ReadData( void ){
    byte ret;
    //   
    WaitBusy();
    // 
    LCD_DATA_PORT=0x00;
    LCD_DATA_DDR=0x00;
    //  
    SetBit(LCD_CONTROL_PORT, A0);
    SetBit(LCD_CONTROL_PORT, RDWR);
    //      
    delay_us(10);
    SetBit(LCD_CONTROL_PORT, E);
    //     
    delay_us(10);
    ret=LCD_DATA_PIN;
    //  
    ClrBit(LCD_CONTROL_PORT,E);
    // 
    return ret;
 }


/*    
      */
void WriteData(byte data){
    WaitBusy();
    LCD_DATA_DDR=0xFF;

    SetBit(LCD_CONTROL_PORT, A0);
    ClrBit(LCD_CONTROL_PORT, RDWR);

    delay_us(10);
    SetBit(LCD_CONTROL_PORT, E);
    LCD_DATA_PORT=data;

    delay_us(10);
    ClrBit(LCD_CONTROL_PORT, E);
}


/*    */
void WriteCode(byte code){
    WaitBusy();
    LCD_DATA_DDR=0xFF;

    ClrBit(LCD_CONTROL_PORT, A0);
    ClrBit(LCD_CONTROL_PORT, RDWR);

    delay_us(10);
    SetBit(LCD_CONTROL_PORT, E);
    LCD_DATA_PORT=code;

    delay_us(10);
    ClrBit(LCD_CONTROL_PORT, E);
}


/*  */
void LCD_INIT( void ){
    byte stat;
    //  Jtag
    MCUCSR = 0x80;
    MCUCSR = 0x80;
    // 
    LCD_DATA_DDR=0xFF;
    LCD_CONTROL_DDR=0xFF;
    //     0.     RES
    LCD_CONTROL_PORT=0x00;
    //    
    delay_ms(100);
    //  .( RES )
    SetBit(LCD_CONTROL_PORT, RES);


    //   

    VR_VF_on;
    V5_regulator;
    electronic_volume_mode_set;
    electronic_volume_register_set;
    static_indicator_off;
    static_indicator_off;
    booster_ratio_select_mode_set;
    step_up;
    duty_bias;
    common_output_mode;
    all_points_not_ON;
    display_not_reverse;
    ADC_select;


    // 1 
    SetBit(LCD_CONTROL_PORT, E1);
    // 
    LCD_ON;
    //  
    LCD_START_LINE(0);

     //   2- 
    ClrBit(LCD_CONTROL_PORT, E1);
    WaitBusy();
    LCD_ON;
    //  
    LCD_START_LINE(0);
}

//  
void LCD_CLS(void){
    byte i;
    byte j;
    byte stat;

    ClrBit(LCD_CONTROL_PORT, E1);
    delay_us(10);

    LCD_OFF;
    LCD_SET_ADDRESS(0);
    for(i=0;i<8; i++){
        LCD_SET_PAGE(i);
        for(j=0; j<64; j++){
            WriteData(0x00);
        }
    }

    LCD_ON;
    SetBit(LCD_CONTROL_PORT, E1);
    delay_us(10);
    LCD_OFF;
    LCD_SET_ADDRESS(0);
    for(i=0;i<8; i++){
        LCD_SET_PAGE(i);
        for(j=0; j<64; j++){
            WriteData(0x00);
        }
    }
    LCD_ON;
}
#include <gl.c>
/*     1 */
void LCD_PUT_PIXEL(byte x, byte y){
    // 
    byte bite;
    byte page;
    byte data, data2;
    //     
    if((x>MAX_X)|(y>MAX_Y)) return;
    // 
    if(x>=64){
        ClrBit(LCD_CONTROL_PORT, E1);
        x=x-64;
        }
        else{
        SetBit(LCD_CONTROL_PORT, E1);
        }
    //page- 
    page=y/8;
    //bite-     
    bite=y%8;
    //   
    LCD_SET_PAGE(page);
    LCD_SET_ADDRESS(x);
    //   LCD(2  ..   )
    data=ReadData();
    data=ReadData();
    //       .
    //    ,    
    switch(met){
        case MET_OR : { data=data|(1<<bite); break;}
        case MET_XOR : {data=data^(1<<bite); break;}
    }
    LCD_SET_ADDRESS(x);
    WriteData(data);
}
/*     .
     ..      */
void LCD_LINE(byte x1, byte y1, byte x2, byte y2){
        int dx, dy, i1, i2, i, kx, ky;
        int d;
        int x, y;
        int flag;

        dy = y2 - y1;
        dx = x2 - x1;
        if (dx == 0 && dy == 0){
                LCD_PUT_PIXEL(x,y);
                return;
        }
        kx = 1;
        ky = 1;

        if( dx < 0 ){ dx = -dx; kx = -1; }
        else if(dx == 0)        kx = 0;

        if(dy < 0) { dy = -dy; ky = -1; }

        if(dx < dy){ flag = 0; d = dx; dx = dy; dy = d; }
        else         flag = 1;

        i1 = dy + dy; d = i1 - dx; i2 = d - dx;
        x = x1; y = y1;

        for(i=0; i < dx; i++){
                LCD_PUT_PIXEL(x,y);

                if(flag) x += kx;
                else     y += ky;

                if( d < 0 )
                         d += i1;
                else{
                         d += i2;
                         if(flag) y += ky;
                         else     x += kx;
                }
        }
        LCD_PUT_PIXEL(x,y);
}

/*     .
    ,  
  sin  cos*/

void LCD_CIRCLE( byte xc, byte yc, byte r )
{
  int d, x, y;
  x=0;
  y=r;
  d=3-2*r; //   !
  LCD_PUT_PIXEL(x+xc,y+yc);
  LCD_PUT_PIXEL(x+xc,-y+yc);
  LCD_PUT_PIXEL(-x+xc,-y+yc);
  LCD_PUT_PIXEL(-x+xc,y+yc);
  LCD_PUT_PIXEL(y+xc,x+yc);
  LCD_PUT_PIXEL(y+xc,-x+yc);
  LCD_PUT_PIXEL(-y+xc,-x+yc);
  LCD_PUT_PIXEL(-y+xc,x+yc);
  while(x<y){
        if(d<=0){
        d=d+4*x+6;
        }else{
        d=d+4*(x-y)+10;
        y--;
        }
  x++;
  //LCD_PUT_PIXEL(x, y);
  LCD_PUT_PIXEL(x+xc,y+yc);
  LCD_PUT_PIXEL(x+xc,-y+yc);
  LCD_PUT_PIXEL(-x+xc,-y+yc);
  LCD_PUT_PIXEL(-x+xc,y+yc);
  LCD_PUT_PIXEL(y+xc,x+yc);
  LCD_PUT_PIXEL(y+xc,-x+yc);
  LCD_PUT_PIXEL(-y+xc,-x+yc);
  LCD_PUT_PIXEL(-y+xc,x+yc);
  }
}
#include <text.c>
/*    */
void LCD_PUT_BYTE(byte x, byte y, byte data){
    //  
    byte tmp_data;
    byte page;
    byte bite;
    byte inv_data;
    //     
    if((x>MAX_X)|(y>MAX_Y)) return;
    //   63   
    if(x>=64){
        ClrBit(LCD_CONTROL_PORT, E1);
        x=x-64;
        }
        else{
        SetBit(LCD_CONTROL_PORT, E1);
        }
    //      
    page=y/8;
    bite=y%8;
    //   (     )
    LCD_SET_PAGE(page);
    LCD_SET_ADDRESS(x);
    // , 2 , ..     
    tmp_data=ReadData();
    tmp_data=ReadData();
    //  
    LCD_SET_PAGE(page);
    LCD_SET_ADDRESS(x);
    //        
    switch(met){
        case MET_OR : {WriteData(tmp_data|(data<<bite)); break;}
        case MET_XOR : {WriteData(tmp_data^(data<<bite)); break;}
        case MET_NOT_OR : {WriteData(tmp_data| ((data^0xFF)<<bite)); break;}
        case MET_NOT_XOR : {WriteData(tmp_data^((data^0xFF)<<bite)); break;}
        }
    //          
    if(bite>0){
        LCD_SET_PAGE(page+1);
        LCD_SET_ADDRESS(x);

        tmp_data=ReadData();
        tmp_data=ReadData();

        LCD_SET_PAGE(page+1);
        LCD_SET_ADDRESS(x);

        switch(met){
            case MET_OR : {WriteData(tmp_data|(data>>(8-bite))); break;}
            case MET_XOR : {WriteData(tmp_data^(data>>(8-bite))); break;}
            case MET_NOT_OR : {WriteData(tmp_data|((data^0xFF)>>(8-bite))); break;}
            case MET_NOT_XOR : {WriteData(tmp_data^((data^0xFF)>>(8-bite))); break;}
        }
    }
}
/*  .     8*5 . */
void LCD_PUTC(byte x, byte y, byte ch){
    byte textL;
    byte i;
    //      ,  
    //   
    if(ch<0x90) textL=0x20;
    else textL=0x60;

    for(i=0; i<5; i++){
        LCD_PUT_BYTE(x+i, y, sym[ch-textL][i]);
        }
    //  
    LCD_PUT_BYTE(x+i, y, 0x00);
}
/*     */
void LCD_PUTSF(byte x, byte y, flash unsigned char str[]){
    byte i=0;
    //     
    while(str[i]){
        LCD_PUTC(x+i*5+i, y, str[i]);
        i++;
    }
}
//========================================================================================================
//   

void main(void)
{

byte i;



//*********************************
        LCD_INIT();


while (1){ //    .

        met=MET_OR;
        LCD_CLS();

        met=MET_NOT_XOR;
        LCD_PUTSF(56,28,"kazus");

        met=MET_XOR;
        LCD_PUTSF(50,50,"kazus");

        delay_ms(1000);

        };
}
